{-

AIF Omega version 2016

Developed under BSD license as part of OFMC

(C) Copyright Alessandro Bruni 2015,2016
(C) Copyright DTU 2010,2016
(C) Copyright Paolo Modesti 2010
(C) Copyright IBM Corp. 2010

All Rights Reserved.

-}

module Main where
import FPASLan
import FPASLanLexer
import FPASLanParser
import AIFOm
import Types
import Modesti
import PrettyPrinter
import System.Environment
import System.Process
import System.IO
import Data.List

data Input = AIF | AIFOm
           deriving (Eq, Show)
data Run = Run { fileName :: String,
                 input :: Input,
                 output :: Output,
                 prove :: Bool }
           deriving (Eq, Show)
defaultOpts = Run { fileName = "", input = AIF, output = Proverif,
                    prove = False }

parseArgs opts (x:xs) = parseArgs (
  case x of
    "-spass" -> opts { output = SPASS }
    "-proverif" -> opts { output = Proverif }
    "-AIFOm" -> opts { input = AIFOm }
    "-AIF" -> opts { input = AIF }
    "-prove" -> opts { prove = True }
    _ -> opts { fileName = x,
                input =
                  if ".aifom" `isSuffixOf` x then AIFOm
                  else if ".aif" `isSuffixOf` x then AIF
                       else (input opts) }
  ) xs
parseArgs opts [] = opts

usage = "Usage: aifomega [options] filename\n\
\    -spass     use the SPASS theorem prover as a backend\n\
\    -proverif  use ProVerif as a backend\n\
\    -AIFOm     force parsing of AIF-Omega file format\n\
\               (default for files ending in .aifom)\n\
\    -AIF       force parsing of AIF file format\n\
\               (default for files ending in .aif)\n\
\    -prove     run the theorem prover"

main = 
  do args <- getArgs
     let opts = parseArgs defaultOpts args
     let prettyPrinter =
           case output opts of
             SPASS -> pASLan
             Proverif -> prASLan
     if (fileName opts) == "" then error usage else return ()
     file <- readFile (fileName opts)
     let paslan = parser . alexScanTokens $ file
     case input opts of
       AIF -> do
         let (paslan@(PASLan name types sets pubfun privfun facts definitions rules)) = mkPrepros paslan
         -- <paolo>--
         if debug then error . dbgDefs . parser . alexScanTokens $ file else putStr ""
         -- </paolo>--
         symbtab <- return $ mkSymbtab paslan
         -- putStr $ Map.showTree symbtab
         putStr $ if spassout then "% Checking rules: "++(show (checkrules symbtab rules))++"\n" else "(* Checking rules: "++(show (checkrules symbtab rules))++"*)\n"
         (rules',numsets) <- return $ absRules symbtab (quantelims symbtab rules)
         putStr (prettyPrinter symbtab numsets name rules')
       AIFOm -> do
         if (output opts /= Proverif)
           then error "Sorry, in this version, the new extensions of AIFOm over AIF are only implemented for ProVerif as prover."
           else return ()
         -- TODO: call normalize to accept AIF's syntactic sugar
         -- TODO: rule checks
         let prolog = compile paslan
         if (prove opts)
           then do
           (tmpPath, h) <- openTempFile "." (fileName opts)
           hPutStr h (pvASLan prolog)
           hClose h
           rawSystem "proverif" [tmpPath]
           rawSystem "rm" [tmpPath]
           return ()
           else do
           putStr (pvASLan prolog)
